// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
//
// "ModuleList" class (and "Module" inner class)
// Responsible for remembering connected modules and handling their data
// Will also automatically control module LEDs/Pumps in the future.
//
package com.teamo.biom.hub;

// ----- Imported Libraries -----
    import java.util.ArrayList;

public class ModuleList
{

    static final int[][] MODULE_TYPE_PRESET = {
    //    LED ON    LED OFF     Humidity 
        { 3*60,     21*60,      450 },      // Type 0: Garden Cress
        { 1*60,     8*60,       400 },      // Type 1: Champignons
        { 10*60,    14*60,      450 },      // Type 2: Strawberries
        { 14*60,    10*60,      450 },      // Type 3: Lettuce
        { 14*60,    10*60,      450 }       // Type 4: Radishes
    };
    static final int MODULE_TYPE_COUNT = MODULE_TYPE_PRESET.length;

    // SENSOR IDs
    static final int SENSOR_COUNT = 2;
    static final int SENSOR_ID_HUMIDITY = 0;
    static final int SENSOR_ID_WATER_TANK = 1;

    static ModuleList ml = new ModuleList();
    static ArrayList<Module> modules = new ArrayList<Module>();     // stores all currently connected modules

    // Development method for testing the app without physical modules. Called in HubServer main thread.
    static void makeTestBoxes()
    {
        modules.add( ml.new Module( 1001, 3, 1 ) );
        modules.add( ml.new Module( 1002, 2, 1 ) );
    }

    // Inner Class "Module", represents a single connected module and contains its current sensor and actuator data.
    private class Module
    {
        // General Fields
            long id;
            int type;
            int[] sensorData;
            int ledPowerOn, ledCycleState;

            long firstCall;

        public Module( long initID, int initType, long initCallId )
        {
            this.id = initID;
            this.type = initType;
            this.sensorData = new int[ SENSOR_COUNT ];
            this.firstCall = initCallId;
        }

        public void renewLastCall( long callId )
        {
            this.firstCall = callId;
        }
    }

    // Method to send actuation commands to the modules via Serial communication. Called upon receiving input from the app in TCPClient interpretData() method.
    static void updateActuatorState( long id, int actId, int actState )
    {
        for ( Module m : modules )
        {
            if ( m.id == id )
            {
                int index = (int)m.id - 1001;
                if ( actId == 0 )
                {
                    m.ledPowerOn = actState;
                    
                    SerialCommunicator.transmit(SerialCommunicator.ports[index], "2");
                } else {
                    SerialCommunicator.transmit(SerialCommunicator.ports[index], "1");
                }
                
            }
        }
    }

    // Method for registering new modules upon detection through the Modcall Protocol.
    static void registerModule( long initId, int initType, long callId )
    {
        boolean isNewModule = true;
        for ( Module m : modules )
        {
            if ( m.id == initId )
            {
                isNewModule = false;
                m.renewLastCall( callId );
            }
        }

        if ( isNewModule )
        {
            modules.add( ml.new Module( initId, initType, callId ) );
            System.out.println( "ModuleList: The Module " + initId + " has been registered." );
        }
    }

    // Method for updating a specific modules sensor data upon receiving input from the module via serial communication.
    static void feedSensorData( long id, int sensorId, int sensorInput )
    {
        for ( Module m : modules )
        {
            if ( m.id == id )
            {
                if ( sensorId < SENSOR_COUNT )
                    m.sensorData[ sensorId ] = sensorInput;
            }
        }
    }

    // Class for cleaning disconnected modules from the module list.
    // Disabled for presentation version!! (needs updating to new modcall system)
    // Runs based on a process timer in controller.
    public class Cleaner extends Thread
    {
        public void run()
        {
            for ( Module m : modules )
            {
                if ( m.firstCall < SerialCommunicator.lastCallId-2 )
                    modules.remove( m );
            }
        }
    }

    // Method to compile all Module information into a String array that the constructProtocol() method can use to build a transmittable message for the app.
    // Used in response to Data Request Protocols sent by the app.
    static String[] getModulePackage()
    {
        ArrayList<String> tempCont = new ArrayList<String>();

        for ( Module m : modules )
        {
            tempCont.add( Long.toString(     m.id ) );
            tempCont.add( Integer.toString(  m.type ) );
            tempCont.add( Integer.toString(  m.ledPowerOn ) );
            tempCont.add( Integer.toString(  m.ledCycleState ) );
            tempCont.add( Integer.toString(  m.sensorData[ SENSOR_ID_HUMIDITY ] ) );
            tempCont.add( Integer.toString(  m.sensorData[ SENSOR_ID_WATER_TANK ] ) );
        }

        String[] content = new String[ tempCont.size() ];
        tempCont.toArray( content );

        return content;
    }

}